Type: Package
Package: BioGA
Title: Bioinformatics Genetic Algorithm (BioGA)
Version: 1.4.0
Authors@R: 
    person("Dany", "Mukesha", , "danymukesha@gmail.com", 
            role = c("aut", "cre"),
           comment = c(ORCID = "0009-0001-9514-751X"))
Description: Genetic algorithm are a class of optimization algorithms
    inspired by the process of natural selection and genetics. This
    package allows users to analyze and optimize high throughput genomic
    data using genetic algorithms.  The functions provided are implemented
    in C++ for improved speed and efficiency, with an easy-to-use
    interface for use within R.
License: MIT + file LICENSE
URL: https://danymukesha.github.io/BioGA/
BugReports: https://github.com/danymukesha/BioGA/issues
Imports: ggplot2, graphics, Rcpp, SummarizedExperiment, animation,
        rlang, biocViews, sessioninfo, BiocStyle
Depends: R (>= 4.4)
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
LinkingTo: Rcpp
VignetteBuilder: knitr
biocViews: ExperimentalDesign, Technology
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.1
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/BioGA
git_branch: RELEASE_3_22
git_last_commit: d779378
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 02:49:26 UTC; biocbuild
Author: Dany Mukesha [aut, cre] (ORCID:
    <https://orcid.org/0009-0001-9514-751X>)
Maintainer: Dany Mukesha <danymukesha@gmail.com>

# Generated by roxygen2: do not edit by hand

export(crossover_cpp)
export(evaluate_fitness_cpp)
export(initialize_population_cpp)
export(mutation_cpp)
export(plot_fitness)
export(plot_fitness_history)
export(plot_population)
export(replacement_cpp)
export(selection_cpp)
importFrom(BiocStyle,html_document)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(animation,saveGIF)
importFrom(biocViews,getBiocViews)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,par)
importFrom(rlang,local_options)
importFrom(sessioninfo,session_info)
useDynLib(BioGA, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->
<!-- badges: start -->

[![R-CMD-check](https://github.com/danymukesha/BioGA/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/danymukesha/BioGA/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

# BioGA <a href="https://danymukesha.github.io/BioGA/"><img src="man/figures/logo.png" align="right" height="139" alt="BioGA website" /></a>

The `BioGA` package provides a set of functions for genetic algorithm
optimization tailored for analyzing high throughput genomic data. These
functions are implemented in C++ for improved speed and efficiency, with
an easy-to-use interface for use within R.

## Installation

You can install the package directly from GitHub using the `devtools`
package:

``` r
devtools::install_github("danymukesha/BioGA")
#> Downloading GitHub repo danymukesha/BioGA@HEAD
#> fs          (1.6.3   -> 1.6.4  ) [CRAN]
#> fastmap     (1.1.1   -> 1.2.0  ) [CRAN]
#> cachem      (1.0.8   -> 1.1.0  ) [CRAN]
#> xfun        (0.42    -> 0.44   ) [CRAN]
#> tinytex     (0.50    -> 0.51   ) [CRAN]
#> knitr       (1.45    -> 1.46   ) [CRAN]
#> htmltools   (0.5.7   -> 0.5.8.1) [CRAN]
#> bslib       (0.6.1   -> 0.7.0  ) [CRAN]
#> rmarkdown   (2.26    -> 2.27   ) [CRAN]
#> matrixStats (1.2.0   -> 1.3.0  ) [CRAN]
#> munsell     (0.5.0   -> 0.5.1  ) [CRAN]
#> farver      (2.1.1   -> 2.1.2  ) [CRAN]
#> BiocManager (1.30.22 -> 1.30.23) [CRAN]
#> bookdown    (0.38    -> 0.39   ) [CRAN]
#> gtable      (0.3.4   -> 0.3.5  ) [CRAN]
#> ggplot2     (3.5.0   -> 3.5.1  ) [CRAN]
#> Skipping 17 packages ahead of CRAN: BiocGenerics, graph, S4Arrays, IRanges, S4Vectors, MatrixGenerics, GenomeInfoDbData, zlibbioc, XVector, GenomeInfoDb, RBGL, Biobase, DelayedArray, GenomicRanges, BiocStyle, biocViews, SummarizedExperiment
#> Installing 16 packages: fs, fastmap, cachem, xfun, tinytex, knitr, htmltools, bslib, rmarkdown, matrixStats, munsell, farver, BiocManager, bookdown, gtable, ggplot2
#> Installing packages into 'C:/Users/dany.mukesha/AppData/Local/Temp/Rtmp63bptc/temp_libpath848868d23488'
#> (as 'lib' is unspecified)
#> Warning: unable to access index for repository https://bioconductor.org/packages/3.17/data/annotation/bin/windows/contrib/4.3:
#>   cannot open URL 'https://bioconductor.org/packages/3.17/data/annotation/bin/windows/contrib/4.3/PACKAGES'
#> Warning: unable to access index for repository https://bioconductor.org/packages/3.17/data/experiment/bin/windows/contrib/4.3:
#>   cannot open URL 'https://bioconductor.org/packages/3.17/data/experiment/bin/windows/contrib/4.3/PACKAGES'
#> Warning: unable to access index for repository https://bioconductor.org/packages/3.17/workflows/bin/windows/contrib/4.3:
#>   cannot open URL 'https://bioconductor.org/packages/3.17/workflows/bin/windows/contrib/4.3/PACKAGES'
#> package 'fs' successfully unpacked and MD5 sums checked
#> package 'fastmap' successfully unpacked and MD5 sums checked
#> package 'cachem' successfully unpacked and MD5 sums checked
#> package 'xfun' successfully unpacked and MD5 sums checked
#> package 'tinytex' successfully unpacked and MD5 sums checked
#> package 'knitr' successfully unpacked and MD5 sums checked
#> package 'htmltools' successfully unpacked and MD5 sums checked
#> package 'bslib' successfully unpacked and MD5 sums checked
#> package 'rmarkdown' successfully unpacked and MD5 sums checked
#> package 'matrixStats' successfully unpacked and MD5 sums checked
#> package 'munsell' successfully unpacked and MD5 sums checked
#> package 'farver' successfully unpacked and MD5 sums checked
#> package 'BiocManager' successfully unpacked and MD5 sums checked
#> package 'bookdown' successfully unpacked and MD5 sums checked
#> package 'gtable' successfully unpacked and MD5 sums checked
#> package 'ggplot2' successfully unpacked and MD5 sums checked
#> 
#> The downloaded binary packages are in
#>  C:\Users\dany.mukesha\AppData\Local\Temp\RtmpArcJMj\downloaded_packages
#> ── R CMD build ─────────────────────────────────────────────────────────────────
#>          checking for file 'C:\Users\dany.mukesha\AppData\Local\Temp\RtmpArcJMj\remotes15e82c92423\danymukesha-BioGA-23ecb91/DESCRIPTION' ...  ✔  checking for file 'C:\Users\dany.mukesha\AppData\Local\Temp\RtmpArcJMj\remotes15e82c92423\danymukesha-BioGA-23ecb91/DESCRIPTION' (343ms)
#>       ─  preparing 'BioGA':
#>    checking DESCRIPTION meta-information ...     checking DESCRIPTION meta-information ...   ✔  checking DESCRIPTION meta-information
#> ─  cleaning src
#>       ─  checking for LF line-endings in source and make files and shell scripts
#>       ─  checking for empty or unneeded directories
#>      Omitted 'LazyData' from DESCRIPTION
#>       ─  building 'BioGA_0.99.5.tar.gz'
#>      
#> 
#> Installing package into 'C:/Users/dany.mukesha/AppData/Local/Temp/Rtmp63bptc/temp_libpath848868d23488'
#> (as 'lib' is unspecified)
```

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)

## ----installation , eval=FALSE------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("BioGA")

## ----installation_from_github , eval=FALSE------------------------------------
# devtools::install_github("danymukesha/BioGA")

## -----------------------------------------------------------------------------
# Load necessary packages
library(BioGA)
library(SummarizedExperiment)

# Define parameters
num_genes <- 1000
num_samples <- 10

# Define parameters for genetic algorithm
population_size <- 100
generations <- 20
mutation_rate <- 0.1

# Generate example genomic data using SummarizedExperiment
counts <- matrix(rpois(num_genes * num_samples, lambda = 10),
    nrow = num_genes
)
rownames(counts) <- paste0("Gene", 1:num_genes)
colnames(counts) <- paste0("Sample", 1:num_samples)

# Create SummarizedExperiment object
se <-
  SummarizedExperiment::SummarizedExperiment(assays = list(counts = counts))

# Convert SummarizedExperiment to matrix for compatibility with BioGA package
genomic_data <- assay(se)

## -----------------------------------------------------------------------------
head(genomic_data)

## -----------------------------------------------------------------------------
# Initialize population (select the number of canditate you wish `population`)
population <- BioGA::initialize_population_cpp(genomic_data,
    population_size = 5
)

## -----------------------------------------------------------------------------
# Initialize fitness history
fitness_history <- list()

# Initialize time progress
start_time <- Sys.time()

# Run genetic algorithm optimization
generation <- 0
while (TRUE) {
    generation <- generation + 1

    # Evaluate fitness
    fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
    fitness_history[[generation]] <- fitness

    # Check termination condition
    if (generation == generations) { # defined number of generations
        break
    }

    # Selection
    selected_parents <- BioGA::selection_cpp(population,
        fitness,
        num_parents = 2
    )

    # Crossover and Mutation
    offspring <- BioGA::crossover_cpp(selected_parents, offspring_size = 2)
    # (no mutation in this example)
    mutated_offspring <- BioGA::mutation_cpp(offspring, mutation_rate = 0)

    # Replacement
    population <- BioGA::replacement_cpp(population, mutated_offspring,
        num_to_replace = 1
    )

    # Calculate time progress
    elapsed_time <- difftime(Sys.time(), start_time, units = "secs")

    # Print time progress
    cat(
        "\rGeneration:", generation, "- Elapsed Time:",
        format(elapsed_time, units = "secs"), "     "
    )
}

## -----------------------------------------------------------------------------
# Plot fitness change over generations
BioGA::plot_fitness_history(fitness_history)

## ----sessioninfo--------------------------------------------------------------
sessioninfo::session_info()

#' Plot Fitness Change Over Generations
#'
#' Plot the change in fitness values over generations.
#'
#' @param fitness_history A list containing fitness values for
#' each generation.
#'
#' @return Plot of fitness history
#'
#' @examples
#' # example of usage
#' fitness_history <- list(c(10, 8, 6, 4, 2), c(9, 7, 5, 3, 1))
#' plot_fitness_history(fitness_history)
#'
#' @export
plot_fitness_history <- function(fitness_history) {
  # Extract fitness values
  fitness_values <- unlist(fitness_history)

  # Create generation index
  generations <-
    rep(
      seq_along(fitness_history),
      vapply(fitness_history, length, integer(1))
    )

  # Create data frame
  df <- data.frame(
    Generation = generations,
    Fitness = fitness_values
  )

  # Plot
  ggplot2::ggplot(df, ggplot2::aes_string(
    x = "Generation",
    y = "Fitness"
  )) +
    geom_line() +
    labs(x = "Generation", y = "Fitness") +
    ggplot2::ggtitle("Fitness Change Over Generations")
}



#' Plot Fitness Values
#'
#' Plot the fitness values of the population over generations.
#'
#' @param fitness_values A numeric vector containing fitness values.
#'
#' @return Plot of fitness
#'
#' @examples
#' # example of usage
#' fitness_values <- c(10, 8, 6, 4, 2)
#' plot_fitness(fitness_values)
#'
#' @export
plot_fitness <- function(fitness_values) {
  generations <- seq_along(fitness_values)
  plot(generations, fitness_values,
       type = "l",
       xlab = "Generations", ylab = "Fitness",
       main = "Fitness Values Over Generations"
  )
}


#' Plot Population Distribution
#'
#' Plot the distribution of individuals in the population.
#'
#' @param population A numeric matrix containing the population data.
#'
#' @return Plot of population
#'
#'
#' @examples
#' # example of usage
#' population <- matrix(runif(100), nrow = 10, ncol = 10)
#' plot_population(population)
#'
#' @export
plot_population <- function(population) {
  par(mfrow = c(1, 2))
  boxplot(population, main = "Boxplot of Population")
  hist(population, main = "Histogram of Population")
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom biocViews getBiocViews
#' @importFrom ggplot2 ggtitle ggplot geom_line labs
#' @importFrom sessioninfo session_info
#' @importFrom SummarizedExperiment SummarizedExperiment
#' @importFrom Rcpp evalCpp sourceCpp
#' @importFrom graphics boxplot hist par
#' @importFrom animation saveGIF
#' @importFrom rlang local_options
#' @importFrom BiocStyle html_document
#' @useDynLib BioGA, .registration = TRUE
## usethis namespace: end
NULL

library(hexSticker)
imgurl <-
    system.file("man/figures/evolution.jpg", package = "BioGA")
sticker(
    imgurl,
    package = "BioGA",
    p_size = 12,
    s_x = 1,
    s_y = .8,
    s_width = .6,
    s_height = .8,
    p_color = "black",
    h_fill = "white",
    h_color = "deepskyblue2",
    filename = "man/figures/logo.png"
)

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Function to perform crossover between selected individuals
#'
#' @param selected_parents Numeric matrix representing the selected
#' individuals.
#' @param offspring_size Number of offspring to generate.
#' @return Numeric matrix representing the offspring.
#' @examples
#' # example of usage
#' genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
#' population <- BioGA::initialize_population_cpp(genomic_data,
#'                 population_size = 5)
#' fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
#' selected_parents <- BioGA::selection_cpp(population, fitness,
#'                 num_parents = 2)
#' BioGA::crossover_cpp(selected_parents, offspring_size = 2)
#' @export
crossover_cpp <- function(selected_parents, offspring_size) {
    .Call(`_BioGA_crossover_cpp`, selected_parents, offspring_size)
}

#' Function to evaluate fitness using genomic data
#'
#' @param genomic_data Numeric matrix of genomic data where rows represent
#' genes/features and columns represent samples.
#' @param population Numeric matrix representing the population of
#' individuals.
#' @return Numeric vector of fitness scores for each individual.
#' @examples
#' # example of usage
#' genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
#' population <- BioGA::initialize_population_cpp(genomic_data,
#'                 population_size = 5)
#' BioGA::evaluate_fitness_cpp(genomic_data, population)
#' @export
evaluate_fitness_cpp <- function(genomic_data, population) {
    .Call(`_BioGA_evaluate_fitness_cpp`, genomic_data, population)
}

#' Function to initialize the population from genomic data
#'
#' @param genomic_data Numeric matrix of genomic data where rows represent
#' genes/features and columns represent samples.
#' @param population_size Number of individuals in the population.
#' @return Numeric matrix representing the initialized population.
#' @examples
#' # example of usage
#' genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
#' BioGA::initialize_population_cpp(genomic_data, population_size = 5)
#' @export
initialize_population_cpp <- function(genomic_data, population_size) {
    .Call(`_BioGA_initialize_population_cpp`, genomic_data, population_size)
}

#' Function to mutate the offspring
#'
#' @param offspring Numeric matrix representing the offspring.
#' @param mutation_rate Probability of mutation for each individual.
#' @return Numeric matrix representing the mutated offspring.
#' @examples
#' # example of usage
#' genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
#' population <- BioGA::initialize_population_cpp(genomic_data,
#'                 population_size = 5)
#' fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
#' selected_parents <- BioGA::selection_cpp(population,
#'                 fitness, num_parents = 2)
#' offspring <- BioGA::crossover_cpp(selected_parents, offspring_size = 2)
#' BioGA::mutation_cpp(offspring, mutation_rate = 0)
#' @export
mutation_cpp <- function(offspring, mutation_rate) {
    .Call(`_BioGA_mutation_cpp`, offspring, mutation_rate)
}

#' Function to replace non-selected individuals in the population
#'
#' Replace non-selected individuals in the population
#' @param population Numeric matrix representing the population of
#' individuals.
#' @param offspring Numeric matrix representing the offspring.
#' @param num_to_replace Number of individuals to replace.
#' @return Numeric matrix representing the updated population.
#' @examples
#' # example of usage
#' genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
#' population <- BioGA::initialize_population_cpp(genomic_data,
#'                 population_size = 5)
#' fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
#' selected_parents <- BioGA::selection_cpp(population, fitness,
#'                       num_parents = 2)
#' offspring <- BioGA::crossover_cpp(selected_parents, offspring_size = 2)
#' mutated_offspring <- BioGA::mutation_cpp(offspring, mutation_rate = 0)
#' BioGA::replacement_cpp(population, mutated_offspring, num_to_replace = 1)
#' @export
replacement_cpp <- function(population, offspring, num_to_replace) {
    .Call(`_BioGA_replacement_cpp`, population, offspring, num_to_replace)
}

#' Function to select individuals based on fitness scores
#'
#' @param population Numeric matrix representing the population of
#' individuals.
#' @param fitness Numeric vector of fitness scores for each individual.
#' @param num_parents Number of individuals to select.
#' @return Numeric matrix representing the selected individuals.
#' @examples
#' # example of usage
#' genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
#' population <- BioGA::initialize_population_cpp(genomic_data,
#'                 population_size = 5)
#' fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
#' BioGA::selection_cpp(population, fitness, num_parents = 2)
#' @export
selection_cpp <- function(population, fitness, num_parents) {
    .Call(`_BioGA_selection_cpp`, population, fitness, num_parents)
}

test_that("selection_cpp returns a matrix with correct dimensions", {
  # Load The PACKAGE
  library(BioGA)
  
  # example of usage
  genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
  population <- BioGA::initialize_population_cpp(genomic_data,
                                                 population_size = 5)
  fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
  selected_population <- BioGA::selection_cpp(population, fitness, 
                                              num_parents = 2)
  
  # Check dimensions
  expect_equal(2, nrow(selected_population))
  
})


test_that("selection_cpp returns a matrix with correct dimensions", {
  # Load The PACKAGE
  library(BioGA)
  
  # example of usage
  genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
  population <- BioGA::initialize_population_cpp(genomic_data,
                                                 population_size = 5)
  fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
  selected_population <- BioGA::selection_cpp(population, fitness, 
                                              num_parents = 2)
  
  testthat::expect_no_warning(BioGA::evaluate_fitness_cpp(genomic_data, 
                                                          population),
                              message = "bananas")
  testthat::expect_no_error(BioGA::evaluate_fitness_cpp(genomic_data, 
                                                        population),
                            message = "bananas")
  
})

test_that("mutation_cpp not produce error or warning", {
  # example of usage
  genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
  population <- BioGA::initialize_population_cpp(genomic_data,
                                                 population_size = 5)
  fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
  selected_parents <- BioGA::selection_cpp(population,
                                           fitness, num_parents = 2)
  offspring <- BioGA::crossover_cpp(selected_parents, offspring_size = 2)
  BioGA::mutation_cpp(offspring, mutation_rate = 0)
  
  testthat::expect_no_warning(BioGA::mutation_cpp(offspring, 
                                                  mutation_rate = 0),
                              message = "bananas")
  
  testthat::expect_no_error(BioGA::mutation_cpp(offspring, 
                                                mutation_rate = 0),
                            message = "bananas")
})

test_that("crossover_cpp not produce error or warning", {
  # Load The PACKAGE
  library(BioGA)
  
  # example of usage
  genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
  population <- BioGA::initialize_population_cpp(genomic_data,
                                                 population_size = 5)
  fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
  selected_parents <- BioGA::selection_cpp(population, fitness,
                                           num_parents = 2)
  BioGA::crossover_cpp(selected_parents, offspring_size = 2)
  
  testthat::expect_no_warning(BioGA::crossover_cpp(selected_parents, 
                                                   offspring_size = 2),
                              message = "bananas")
  
  testthat::expect_no_error(BioGA::crossover_cpp(selected_parents, 
                                                   offspring_size = 2),
                              message = "bananas")
})

test_that("evalutate_cpp not produce error or warning", {
  # example of usage
  genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
  population <- BioGA::initialize_population_cpp(genomic_data,
                                                 population_size = 5)
  
  output <- tryCatch(
    BioGA::evaluate_fitness_cpp(genomic_data, population), 
    error = function(e) return(NA)
  )
  
  testthat::expect_no_warning(BioGA::evaluate_fitness_cpp(genomic_data, 
                                                          population),
                              message = "bananas")
  testthat::expect_no_error(BioGA::evaluate_fitness_cpp(genomic_data, 
                                                          population),
                              message = "bananas")
})

test_that("initialize_population_cpp returns a matrix 
          with correct dimensions", {
    # Load The PACKAGE
    library(BioGA)

    # Generate some test data
    genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
    population_size <- 5

    # Call the function
    population <- initialize_population_cpp(genomic_data, population_size)

    # Check dimensions
    expect_equal(nrow(population), population_size)
    expect_equal(ncol(population), nrow(genomic_data))
})

test_that("initialize_population_cpp returns a matrix 
          with values from genomic data", {
    # Load THE PACKAGE
    library(BioGA)

    # Generate some test data
    genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
    population_size <- 5

    # Call the function
    population <- initialize_population_cpp(genomic_data, population_size)

    # Check if all values in the population are present in genomic data
    for (i in 1:nrow(population)) {
        for (j in 1:ncol(population)) {
            expect_true(population[i, j] %in% genomic_data[j, ])
        }
    }
})

---
title: "Introduction"
author: "Dany Mukesha"
date: "`r Sys.Date()`"
abstract:
    Genetic algorithms (GAs) are optimization techniques inspired by 
    the process of natural selection and genetics. They operate by evolving 
    a population of candidate solutions over successive generations, 
    with each individual representing a potential solution to the optimization
    problem at hand. Through the application of genetic operators 
    such as selection, crossover, and mutation, genetic algorithms 
    iteratively improve the population, eventually converging towards 
    optimal or near-optimal solutions.
    
    In the field of genomics, where data sets are often large, complex, 
    and high-dimensional, genetic algorithms offer a good approach 
    for addressing optimization challenges such as feature selection, 
    parameter tuning, and model optimization. By harnessing the power 
    of evolutionary principles, genetic algorithms can effectively explore 
    the solution space, identify informative features, and optimize 
    model parameters, leading to improved accuracy and interpretability 
    in genomic data analysis.
    
    The BioGA package extends the capabilities of genetic algorithms 
    to the realm of genomic data analysis, providing a suite of functions 
    optimized for handling high throughput genomic data. Implemented in C++ 
    for enhanced performance, BioGA offers efficient algorithms for tasks 
    such as feature selection, classification, clustering, and more. 
    By integrating seamlessly with the Bioconductor ecosystem, 
    BioGA empowers researchers and analysts to leverage the power 
    of genetic algorithms within their genomics workflows, facilitating 
    the discovery of biological insights from large-scale genomic data sets.
output: 
    BiocStyle::html_document
vignette: >
  %\VignetteIndexEntry{Introduction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```

<br>

# Getting Started

## Installation

To install this package, start R (version "4.4") and enter:

```{r installation , eval=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("BioGA")
```

You can also install the package directly from GitHub 
using the `devtools` package:

```{r installation_from_github , eval=FALSE}
devtools::install_github("danymukesha/BioGA")
```

In this vignette, we illustrate the usage of BioGA for genetic algorithm 
optimization in the context of high throughput genomic data analysis. 
We showcase its interoperability with Bioconductor classes, demonstrating
how genetic algorithm optimization can be seamlessly integrated into 
existing genomics pipelines for improved analysis and interpretation.

The BioGA package provides a  set of functions for genetic algorithm 
optimization tailored for analyzing high throughput genomic data. 
This vignette demonstrates the usage of BioGA in the context of selecting
the best combination of genes for predicting a certain trait, 
such as disease susceptibility.

## Overview

Genomic data refers to the genetic information stored in an organism's DNA. 
It includes the sequence of nucleotides (adenine, thymine, cytosine, 
and guanine) that make up the DNA molecules. Genomic data can provide valuable 
insights into various biological processes, such as gene expression, 
genetic variation, and evolutionary relationships.

Genomic data in this context could consist of gene expression profiles 
measured across different individuals (e.g., patients).

- Each row in the genomic_data matrix represents a gene, and each column 
represents a patient sample.

- The values in the matrix represent the expression levels of each gene in 
each patient sample.

Here's an example of genomic data:

```
      Sample 1   Sample 2   Sample 3   Sample 4
Gene1    0.1        0.2        0.3        0.4
Gene2    1.2        1.3        1.4        1.5
Gene3    2.3        2.2        2.1        2.0
```

In this example, each row represents a gene (or genomic feature), and 
each column represents a sample. The values in the matrix represent 
some measurement of gene expression, such as mRNA levels or protein abundance,
in each sample.

For instance, the value 0.1 in Sample 1 for Gene1 indicates the expression 
level of Gene1 in Sample 1. Similarly, the value 2.2 in Sample 2 for Gene3 
indicates the expression level of Gene3 in Sample 2.

Genomic data can be used in various analyses, including genetic association 
studies, gene expression analysis, and comparative genomics. In the context 
of the `evaluate_fitness_cpp` function, genomic data is used to calculate 
fitness scores for individuals in a population, typically in the context 
of genetic algorithm optimization.

The population represents a set of candidate combinations of genes that 
could be predictive of the trait.
Each individual in the population is represented by a binary vector indicating
the presence or absence of each gene.
For example, an individual in the population might be represented as 
[1, 0, 1],
indicating the presence of Gene1 and Gene3 but the absence of Gene2.
The population undergoes genetic algorithm operations such as selection, 
crossover, mutation, and replacement to evolve towards individuals with higher
predictive power for the trait.

## Example Scenario 

Consider an example scenario of using genetic algorithm optimization to select
the best combination of genes for predicting a certain trait, such as disease
susceptibility.

```{r}
# Load necessary packages
library(BioGA)
library(SummarizedExperiment)

# Define parameters
num_genes <- 1000
num_samples <- 10

# Define parameters for genetic algorithm
population_size <- 100
generations <- 20
mutation_rate <- 0.1

# Generate example genomic data using SummarizedExperiment
counts <- matrix(rpois(num_genes * num_samples, lambda = 10),
    nrow = num_genes
)
rownames(counts) <- paste0("Gene", 1:num_genes)
colnames(counts) <- paste0("Sample", 1:num_samples)

# Create SummarizedExperiment object
se <-
  SummarizedExperiment::SummarizedExperiment(assays = list(counts = counts))

# Convert SummarizedExperiment to matrix for compatibility with BioGA package
genomic_data <- assay(se)
```

In this example, `counts` is a matrix representing the counts 
of gene expression levels across different samples. Each row corresponds 
to a gene, and each column corresponds to a sample. We use the 
`SummarizedExperiment` class to store this data, which is 
common Bioconductor class for representing rectangular feature x sample data,
such as RNAseq count matrices or microarray data.

```{r}
head(genomic_data)
```

## Initialization

```{r}
# Initialize population (select the number of canditate you wish `population`)
population <- BioGA::initialize_population_cpp(genomic_data,
    population_size = 5
)
```

The population represents a set of candidate combinations of genes that 
could be predictive of the trait. Each individual in the population is 
represented by a binary vector indicating the presence or absence of 
each gene.
For example, an individual in the population might be represented 
as [1, 0, 1], indicating the presence of Gene1 and Gene3 but the absence
of Gene2. 
The population undergoes genetic algorithm operations such as selection, 
crossover, mutation, and replacement to evolve towards individuals 
with higher predictive power for the trait.

## Genetic Algorithm Optimization

```{r}
# Initialize fitness history
fitness_history <- list()

# Initialize time progress
start_time <- Sys.time()

# Run genetic algorithm optimization
generation <- 0
while (TRUE) {
    generation <- generation + 1

    # Evaluate fitness
    fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
    fitness_history[[generation]] <- fitness

    # Check termination condition
    if (generation == generations) { # defined number of generations
        break
    }

    # Selection
    selected_parents <- BioGA::selection_cpp(population,
        fitness,
        num_parents = 2
    )

    # Crossover and Mutation
    offspring <- BioGA::crossover_cpp(selected_parents, offspring_size = 2)
    # (no mutation in this example)
    mutated_offspring <- BioGA::mutation_cpp(offspring, mutation_rate = 0)

    # Replacement
    population <- BioGA::replacement_cpp(population, mutated_offspring,
        num_to_replace = 1
    )

    # Calculate time progress
    elapsed_time <- difftime(Sys.time(), start_time, units = "secs")

    # Print time progress
    cat(
        "\rGeneration:", generation, "- Elapsed Time:",
        format(elapsed_time, units = "secs"), "     "
    )
}
```

## Fitness Calculation

The fitness calculation described in the provided code calculates a measure 
of dissimilarity between the gene expression profiles of individuals 
in the population and the genomic data. This measure of dissimilarity, 
or "fitness", quantifies how well the gene expression profile of an individual
matches the genomic data.

Mathematically, the fitness calculation can be represented as follows:

Let:

- \( g_{ijk} \) be the gene expression level of gene \( j \) 
in individual \( i \) and sample \( k \) from the genomic data.

- \( p_{ij} \) be the gene expression level of gene \( j \) 
in individual \( i \) from the population.

- \( N \) be the number of individuals in the population.

- \( G \) be the number of genes.

- \( S \) be the number of samples.

Then, the fitness \( F_i \) for individual \( i \) in the population can be 
calculated as the sum of squared differences between the gene expression 
levels of individual \( i \) and the corresponding gene expression levels 
in the genomic data, across all genes and samples:
$$
 F_i = \sum_{j=1}^{G} \sum_{k=1}^{S} (g_{ijk} - p_{ij})^2 
$$

This fitness calculation aims to minimize the overall dissimilarity between 
the gene expression profiles of individuals in the population and 
the genomic data. Individuals with lower fitness scores are considered to have
gene expression profiles that are more similar to the genomic data and 
are therefore more likely to be selected for further optimization 
in the genetic algorithm.

```{r}
# Plot fitness change over generations
BioGA::plot_fitness_history(fitness_history)
```

This showcases the integration of genetic algorithms with genomic 
data analysis and highlights the potential of genetic algorithms
for feature selection in genomics.

Here's how BioGA could work in the context of high throughput genomic data 
analysis:

1. **Problem Definition**: BioGA starts with a clear definition of the 
problem to be solved. This could include tasks such as identifying genetic 
markers associated with a particular disease, optimizing gene expression 
patterns, or clustering genomic data to identify patterns or groupings.

2. **Representation**: Genomic data would need to be appropriately represented
for use within the genetic algorithm framework. This might involve encoding 
the data in a suitable format, such as binary strings representing genes 
or chromosomes.

3. **Fitness Evaluation**: BioGA would define a fitness function that 
evaluates how well a particular solution performs with respect to the 
problem being addressed. In the context of genomic data analysis, 
this could involve measures such as classification accuracy, correlation
with clinical outcomes, or fitness to a particular model.

4. **Initialization**: The algorithm would initialize a population of 
candidate solutions, typically randomly or using some heuristic method. 
Each solution in the population represents a potential solution 
to the problem at hand.

5. **Genetic Operations**: BioGA would apply genetic operators such as 
selection, crossover, and mutation to evolve the population over successive
generations. Selection identifies individuals with higher fitness to serve
as parents for the next generation. Crossover combines genetic material
from two parent solutions to produce offspring. Mutation introduces
random changes to the offspring to maintain genetic diversity.

6. **Termination Criteria**: The algorithm would continue iterating through
generations until a termination criterion is met. This could be a maximum
number of generations, reaching a satisfactory solution, or convergence
of the population.

7. **Result Analysis**: Once the algorithm terminates, BioGA would analyze
the final population to identify the best solution(s) found. This could
involve further validation or interpretation of the results in the context
of the original problem.

Other applications of BioGA in genomic data analysis could include genome-wide
association studies (GWAS), gene expression analysis, pathway analysis,
and predictive modeling for personalized medicine, among others.
By leveraging genetic algorithms, BioGA offers a powerful approach
to exploring complex genomic datasets and identifying meaningful patterns
and associations.


<details>

<summary>**Session Info**</summary>

```{r sessioninfo}
sessioninfo::session_info()
```

</details>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{crossover_cpp}
\alias{crossover_cpp}
\title{Function to perform crossover between selected individuals}
\usage{
crossover_cpp(selected_parents, offspring_size)
}
\arguments{
\item{selected_parents}{Numeric matrix representing the selected
individuals.}

\item{offspring_size}{Number of offspring to generate.}
}
\value{
Numeric matrix representing the offspring.
}
\description{
Function to perform crossover between selected individuals
}
\examples{
# example of usage
genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
population <- BioGA::initialize_population_cpp(genomic_data,
                population_size = 5)
fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
selected_parents <- BioGA::selection_cpp(population, fitness,
                num_parents = 2)
BioGA::crossover_cpp(selected_parents, offspring_size = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{mutation_cpp}
\alias{mutation_cpp}
\title{Function to mutate the offspring}
\usage{
mutation_cpp(offspring, mutation_rate)
}
\arguments{
\item{offspring}{Numeric matrix representing the offspring.}

\item{mutation_rate}{Probability of mutation for each individual.}
}
\value{
Numeric matrix representing the mutated offspring.
}
\description{
Function to mutate the offspring
}
\examples{
# example of usage
genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
population <- BioGA::initialize_population_cpp(genomic_data,
                population_size = 5)
fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
selected_parents <- BioGA::selection_cpp(population,
                fitness, num_parents = 2)
offspring <- BioGA::crossover_cpp(selected_parents, offspring_size = 2)
BioGA::mutation_cpp(offspring, mutation_rate = 0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Plottings.R
\name{plot_fitness_history}
\alias{plot_fitness_history}
\title{Plot Fitness Change Over Generations}
\usage{
plot_fitness_history(fitness_history)
}
\arguments{
\item{fitness_history}{A list containing fitness values for
each generation.}
}
\value{
Plot of fitness history
}
\description{
Plot the change in fitness values over generations.
}
\examples{
# example of usage
fitness_history <- list(c(10, 8, 6, 4, 2), c(9, 7, 5, 3, 1))
plot_fitness_history(fitness_history)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Plottings.R
\name{plot_fitness}
\alias{plot_fitness}
\title{Plot Fitness Values}
\usage{
plot_fitness(fitness_values)
}
\arguments{
\item{fitness_values}{A numeric vector containing fitness values.}
}
\value{
Plot of fitness
}
\description{
Plot the fitness values of the population over generations.
}
\examples{
# example of usage
fitness_values <- c(10, 8, 6, 4, 2)
plot_fitness(fitness_values)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{replacement_cpp}
\alias{replacement_cpp}
\title{Function to replace non-selected individuals in the population}
\usage{
replacement_cpp(population, offspring, num_to_replace)
}
\arguments{
\item{population}{Numeric matrix representing the population of
individuals.}

\item{offspring}{Numeric matrix representing the offspring.}

\item{num_to_replace}{Number of individuals to replace.}
}
\value{
Numeric matrix representing the updated population.
}
\description{
Replace non-selected individuals in the population
}
\examples{
# example of usage
genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
population <- BioGA::initialize_population_cpp(genomic_data,
                population_size = 5)
fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
selected_parents <- BioGA::selection_cpp(population, fitness,
                      num_parents = 2)
offspring <- BioGA::crossover_cpp(selected_parents, offspring_size = 2)
mutated_offspring <- BioGA::mutation_cpp(offspring, mutation_rate = 0)
BioGA::replacement_cpp(population, mutated_offspring, num_to_replace = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BioGA-package.R
\docType{package}
\name{BioGA-package}
\alias{BioGA}
\alias{BioGA-package}
\title{BioGA: Bioinformatics Genetic Algorithm (BioGA)}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Genetic algorithm are a class of optimization algorithms inspired by the process of natural selection and genetics. This package allows users to analyze and optimize high throughput genomic data using genetic algorithms. The functions provided are implemented in C++ for improved speed and efficiency, with an easy-to-use interface for use within R.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://danymukesha.github.io/BioGA/}
  \item Report bugs at \url{https://github.com/danymukesha/BioGA/issues}
}

}
\author{
\strong{Maintainer}: Dany Mukesha \email{danymukesha@gmail.com} (\href{https://orcid.org/0009-0001-9514-751X}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{initialize_population_cpp}
\alias{initialize_population_cpp}
\title{Function to initialize the population from genomic data}
\usage{
initialize_population_cpp(genomic_data, population_size)
}
\arguments{
\item{genomic_data}{Numeric matrix of genomic data where rows represent
genes/features and columns represent samples.}

\item{population_size}{Number of individuals in the population.}
}
\value{
Numeric matrix representing the initialized population.
}
\description{
Function to initialize the population from genomic data
}
\examples{
# example of usage
genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
BioGA::initialize_population_cpp(genomic_data, population_size = 5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{evaluate_fitness_cpp}
\alias{evaluate_fitness_cpp}
\title{Function to evaluate fitness using genomic data}
\usage{
evaluate_fitness_cpp(genomic_data, population)
}
\arguments{
\item{genomic_data}{Numeric matrix of genomic data where rows represent
genes/features and columns represent samples.}

\item{population}{Numeric matrix representing the population of
individuals.}
}
\value{
Numeric vector of fitness scores for each individual.
}
\description{
Function to evaluate fitness using genomic data
}
\examples{
# example of usage
genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
population <- BioGA::initialize_population_cpp(genomic_data,
                population_size = 5)
BioGA::evaluate_fitness_cpp(genomic_data, population)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Plottings.R
\name{plot_population}
\alias{plot_population}
\title{Plot Population Distribution}
\usage{
plot_population(population)
}
\arguments{
\item{population}{A numeric matrix containing the population data.}
}
\value{
Plot of population
}
\description{
Plot the distribution of individuals in the population.
}
\examples{
# example of usage
population <- matrix(runif(100), nrow = 10, ncol = 10)
plot_population(population)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{selection_cpp}
\alias{selection_cpp}
\title{Function to select individuals based on fitness scores}
\usage{
selection_cpp(population, fitness, num_parents)
}
\arguments{
\item{population}{Numeric matrix representing the population of
individuals.}

\item{fitness}{Numeric vector of fitness scores for each individual.}

\item{num_parents}{Number of individuals to select.}
}
\value{
Numeric matrix representing the selected individuals.
}
\description{
Function to select individuals based on fitness scores
}
\examples{
# example of usage
genomic_data <- matrix(rnorm(100), nrow = 10, ncol = 10)
population <- BioGA::initialize_population_cpp(genomic_data,
                population_size = 5)
fitness <- BioGA::evaluate_fitness_cpp(genomic_data, population)
BioGA::selection_cpp(population, fitness, num_parents = 2)
}
